@@ -56,8 +56,8 @@ static void gpm_killed(int);
 struct mouse_features mouse_table[3] = {
   {
   DEF_TYPE, DEF_DEV, DEF_SEQUENCE,
-  DEF_BAUD, DEF_SAMPLE, DEF_DELTA, DEF_ACCEL, DEF_SCALE,
-  DEF_TIME, DEF_CLUSTER, DEF_THREE, DEF_TOGGLE,
+  DEF_BAUD, DEF_SAMPLE, DEF_DELTA, DEF_ACCEL, DEF_SCALE, 0 /* scaley */,
+  DEF_TIME, DEF_CLUSTER, DEF_THREE, DEF_TOGGLE, DEF_GLIDEPOINT_TAP,
   (Gpm_Type *)NULL
   },
 };
@@ -65,6 +65,7 @@ struct mouse_features *which_mouse;
 
 /* These are only the 'global' options */
 
+int opt_quit=0;
 char *opt_lut=DEF_LUT;
 int opt_test=DEF_TEST;
 int opt_ptrdrag=DEF_PTRDRAG;
@@ -241,7 +242,7 @@ MAGIC_P((static int magic=GPM_MAGIC));
   if ((info.maxMod & event->modifiers) < event->modifiers) return 0;
 
   /* not managed, use default mask */
-  if (!(info.eventMask & event->type))
+  if (!(info.eventMask & GPM_BARE_EVENTS(event->type)))
     return res;
  
   /* WARNING */ /* This can generate a SIGPIPE... I'd better catch it */
@@ -321,7 +322,7 @@ static inline int processMouse(int fd, Gpm_Event *event, Gpm_Type *type,
 {
 char *data;
 static int fine_dx, fine_dy;
-static int i, margin;
+static int i, m;
 static Gpm_Event nEvent;
 static struct vt_stat stat;
 static struct timeval tv1={0,0}, tv2; /* tv1=0: force first click as single */
@@ -340,6 +341,17 @@ static int newB=0, oldB=0, oldT=0; /* old buttons and Type to chain events */
   if (eventFlag)
     {
     eventFlag=0;
+
+    if (!(m_type->absolute))
+      {
+      event->dx=nEvent.dx;
+      event->dy=nEvent.dy;
+      }
+    else /* absolute: a pen */
+      {
+      event->x=nEvent.x;
+      event->y=nEvent.y;
+      }
     event->dx=nEvent.dx;
     event->dy=nEvent.dy;
     event->buttons=nEvent.buttons;
@@ -367,12 +379,21 @@ static int newB=0, oldB=0, oldT=0; /* old buttons and Type to chain events */
 	break;
 	}
 
-      if (abs(nEvent.dx)+abs(nEvent.dy) > opt_delta)
-	nEvent.dx*=opt_accel, nEvent.dy*=opt_accel;
+      if (!(m_type->absolute)) /* mouse */
+	{
+	if (abs(nEvent.dx)+abs(nEvent.dy) > opt_delta)
+	  nEvent.dx*=opt_accel, nEvent.dy*=opt_accel;
 
-      /* increment the reported dx,dy */
-      event->dx+=nEvent.dx;
-      event->dy+=nEvent.dy;
+	/* increment the reported dx,dy */
+	event->dx+=nEvent.dx;
+	event->dy+=nEvent.dy;
+	}
+      else /* a pen */
+	{
+        /* get dx,dy to check if there has been movement */
+        event->dx = (nEvent.x) - (event->x);
+        event->dy = (nEvent.y) - (event->y);
+	}
 
       select(fd+1,&fdSet,(fd_set *)NULL,(fd_set *)NULL,&timeout/* zero */);
       }
@@ -403,13 +424,27 @@ static int newB=0, oldB=0, oldT=0; /* old buttons and Type to chain events */
 
 /*....................................... no, we arent a repeater, go on */
 
-  /* use fine values now */
-  fine_dx+=event->dx;             fine_dy+=event->dy;
-  event->dx=fine_dx/opt_scale;    event->dy=fine_dy/opt_scale;
-  fine_dx %= opt_scale;           fine_dy %= opt_scale;
-  if (!event->dx && !event->dy && (event->buttons==oldB)) 
-    return 0;
+  /* use fine delta values now, if delta is the information */
+  if (!(m_type)->absolute)
+    {
+    fine_dx+=event->dx;             fine_dy+=event->dy;
+    event->dx=fine_dx/opt_scale;    event->dy=fine_dy/opt_scaley;
+    fine_dx %= opt_scale;           fine_dy %= opt_scaley;
+    }
 
+  if (!event->dx && !event->dy && (event->buttons==oldB))
+    do 
+      { /* so to break */
+      static long awaketime;
+      /*
+       * Return information also if never happens, but enough time has elapsed.
+       * Note: return 1 will segfault due to missing event->vc
+       */
+      if (time(NULL)<=awaketime) return 0;
+      awaketime=time(NULL)+1;
+      break;
+      }
+    while (0);
 
 /*....................................... fill missing fields */
 
@@ -436,16 +471,19 @@ static int newB=0, oldB=0, oldT=0; /* old buttons and Type to chain events */
       if (tv1.tv_sec && (DIF_TIME(tv1,tv2)<opt_time)) /* check first click */
 	statusC++, statusC%=3; /* 0, 1 or 2 */
       else statusC=0;
-      break;
+      event->type|=(GPM_SINGLE<<statusC);
+    break;
 
     case GPM_UP:
       GET_TIME(tv1);
       event->buttons^=oldB; /* for button-up, tell which one */
       event->type|= (oldT&GPM_MFLAG);
+      event->type|=(GPM_SINGLE<<statusC);
       break;
 
     case GPM_DRAG:
       event->type |= GPM_MFLAG;
+      event->type|=(GPM_SINGLE<<statusC);
       break;
 
     case GPM_MOVE:
@@ -454,25 +492,31 @@ static int newB=0, oldB=0, oldT=0; /* old buttons and Type to chain events */
       break;
     }
   event->clicks=statusC;
-  event->type|=(GPM_SINGLE<<statusC);
-
+  
 /* UGLY */
 /* The current policy is to force the following behaviour:
  * - At buttons up, must fit inside the screen, though flags are set.
  * - At button down, allow going outside by one single step
  */
 
-  margin=0;
+
+  /* selection used 1-based coordinates, so do I */
+
+  /*
+   * 1.05: only one margin is current. Y takes priority over X.
+   * The i variable is how much margin is allowed. "m" is which one is there.
+   */
+
+  m = 0;
   i = ((event->type&(GPM_DRAG|GPM_UP))!=0); /* i is boolean */
 
-  /* Hmm... seems that selection used 1-based coordinates... */
+  if (event->y>win.ws_row)  {event->y=win.ws_row+1-!i; i=0; m = GPM_BOT;}
+  else if (event->y<=0)     {event->y=1-i;             i=0; m = GPM_TOP;}
 
-  if (event->x>win.ws_col)  {margin = GPM_RGT; event->x=win.ws_col+1-!i;}
-  else if (event->x<=0)     {margin = GPM_LFT; event->x=1-i;}
-  if (event->y>win.ws_row)  {margin = GPM_BOT; event->y=win.ws_row+1-!i;}
-  else if (event->y<=0)     {margin = GPM_TOP; event->y=1-i;}
+  if (event->x>win.ws_col)  {event->x=win.ws_col+1-!i; if (!m) m = GPM_RGT;}
+  else if (event->x<=0)     {event->x=1-i;             if (!m) m = GPM_LFT;}
 
-  event->margin=margin;
+  event->margin=m;
 
   LOG(("M: %3i %3i (%3i %3i) - butt=%i vc=%i cl=%i",event->dx,event->dy,
                                       event->x,event->y,
@@ -485,7 +529,10 @@ static int newB=0, oldB=0, oldT=0; /* old buttons and Type to chain events */
 }
 
 /*-------------------------------------------------------------------*/
-static inline int get_data(Gpm_Connect *where, int whence)
+/*
+ * This  was inline, and incurred in a compiler bug (2.7.0)
+ */
+static int get_data(Gpm_Connect *where, int whence)
 {
 static int i;
 
@@ -522,6 +569,7 @@ static Gpm_Event event;
 static struct vt_stat stat;
 
   PDEBUGG((stderr,"Request on %i (console %i)\n",fd,vc));
+  if (vc>MAX_VC) return -1;
 
   /* Hmm... data on inactive client: drop him */
   if (vc==-1)                   
@@ -581,11 +629,17 @@ static struct vt_stat stat;
         event.dx=maxx;     event.dy=maxy;
         event.buttons= statusB;
         event.clicks=statusC;
+	/* fall through */
 
     case GPM_REQ_BUTTONS:
         event.type= (opt_three==1 ? 3 : 2); /* buttons */
         write(fd,&event,sizeof(Gpm_Event));
-      }
+	break;
+
+    case GPM_REQ_CONFIG:
+	xfer_options(1,fd);
+	break;
+    }
 
   return 0;
 }
@@ -636,7 +690,7 @@ struct sockaddr_un addr; /* reuse this each time */
   info->fd=newfd;
   cinfo[vc]=info;
 
-  LOG(("Pid %i, vc %i, events %02X, default %02X, minmod %02X, maxmod %02X",
+  PDEBUG((stderr,"Pid %i, vc %i, ev %02X, def %02X, minm %02X, maxm %02X\n",
        request->pid, request->vc, request->eventMask, request->defaultMask,
        request->minMod, request->maxMod));
 
@@ -654,6 +708,8 @@ struct sockaddr_un addr; /* reuse this each time */
 void get_console_size(Gpm_Event *ePtr)
 {
 int i;
+struct mouse_features *which_mouse; /* local */
+
   i=open_console(O_RDONLY);
   ioctl(i, TIOCGWINSZ, &win);
   close(i);
@@ -665,7 +721,17 @@ int i;
   maxx=win.ws_col; maxy=win.ws_row;
   PDEBUGG((stderr,"%i - %i\n",maxx,maxy));
   statusX=ePtr->x=maxx/2; statusY=ePtr->y=maxy/2;
-  
+
+  for (i=1; i <= 1+opt_double; i++)
+    {
+    which_mouse=mouse_table+i; /* used to access options */
+    /*
+     * the following operation is based on the observation that 80x50
+     * has square cells. (An author-centric observation ;-)
+     */
+    opt_scaley=opt_scale*50*maxx/80/maxy;
+    PDEBUG((stderr,"x %i, y %i\n",opt_scale,opt_scaley));
+    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -820,7 +886,7 @@ Gpm_Event event;
 	     * pass it to the client, if any
 	     * or to the default handler, if any
 	     * or to the selection handler
-	     */
+	     */ /* FIXME -- check event.vc */
 	    (cinfo[event.vc] && do_client(cinfo[event.vc], &event))
 	       || (cinfo[0]        && do_client(cinfo[0],        &event))
 	       ||  do_selection(&event);
@@ -891,12 +957,4 @@ static void gpm_killed(int signo)
 }
 
 
-/*
- * This is because Linus uses 4-wide tabstops,
- * forcing me to use the same default to manage kernel sources
- */
-
-/* Local Variables: */
-/* tab-width:8      */
-/* End:             */
 
