@@ -19,8 +19,8 @@
  ********/
 
 #define GPM_NAME    "gpm-Linux"
-#define GPM_RELEASE "1.00"
-#define GPM_DATE    "July 1995"
+#define GPM_RELEASE "1.06"
+#define GPM_DATE    "November 1995"
 
 /* You can define DEBUG, or undefine it. */
 /* Well, better putting it in the Makefile... */
@@ -30,11 +30,10 @@
 #undef CONFIG_GPM_LOG
 
 /* timeout for the select() syscall */
-/* it's short to check the graphic mode */
-#define SELECT_TIME 60
+#define SELECT_TIME 600
 
 /* How many vitual consoles are managed? */
-#define MAX_VC    16
+#define MAX_VC    32
 
 /* How many buttons may the mouse have? */
 /* #define MAX_BUTTONS 3  ===> not used, it is hardwired :-( */
@@ -56,3 +55,4 @@
 #define DEF_TEST             0
 #define DEF_PTRDRAG          1    /* double or triple click */
 #define DEF_TOGGLE           0
+#define DEF_GLIDEPOINT_TAP   1    /* tapping emulates the first button */
