@@ -22,9 +22,13 @@
 #include "gpm.h"
 
 #if !defined(__GNUC__)
-#define inline
+#  define inline
 #endif 
 
+#ifdef DEBUG
+#  define inline
+#endif
+
 #include "wd.h" /* when debugging macros */
 
 /*....................................... Logging */
@@ -41,6 +45,7 @@ extern int Log(char *fmt, ...);
 
 #define GPM_REQ_SNAPSHOT 0
 #define GPM_REQ_BUTTONS  1
+#define GPM_REQ_CONFIG   2
 
 /*....................................... Structures */
 
@@ -60,6 +65,7 @@ typedef struct Gpm_Type {
   int packetlen;
   int howmany;            /* howmany bytes to read at a time */
   int getextra;           /* does it get an extra byte? (only mouseman) */
+  int absolute;           /* flag indicating absolute pointing device */
 }                   Gpm_Type;
 
 #define GPM_EXTRA_MAGIC_1 0xAA
@@ -78,8 +84,8 @@ typedef struct Gpm_Cinfo {
 
 struct mouse_features {
   char *opt_type, *opt_dev, *opt_sequence;
-  int opt_baud,opt_sample,opt_delta, opt_accel, opt_scale;
-  int opt_time, opt_cluster, opt_three, opt_toggle;
+  int opt_baud,opt_sample,opt_delta, opt_accel, opt_scale, opt_scaley;
+  int opt_time, opt_cluster, opt_three, opt_toggle, opt_glidepoint_tap;
   Gpm_Type *m_type;
   int fd;
 };
@@ -94,16 +100,20 @@ extern struct mouse_features mouse_table[3], *which_mouse; /*the current one*/
 #define opt_delta    (which_mouse->opt_delta)
 #define opt_accel    (which_mouse->opt_accel)
 #define opt_scale    (which_mouse->opt_scale)
+#define opt_scaley   (which_mouse->opt_scaley)
 #define opt_time     (which_mouse->opt_time)
 #define opt_cluster  (which_mouse->opt_cluster)
 #define opt_three    (which_mouse->opt_three)
 #define opt_toggle   (which_mouse->opt_toggle)
+#define opt_glidepoint_tap \
+                     (which_mouse->opt_glidepoint_tap)
 
 #define m_type       (which_mouse->m_type)
 
 /* the other variables */
 
 extern char *prgname;
+extern int opt_quit;
 extern char *opt_lut;
 extern int opt_test, opt_ptrdrag;
 extern int opt_kill;
@@ -122,6 +132,7 @@ int main(int argc, char **argv);
        /* gpn.c */
 int Log(char *fmt,...);              /* the logging utility               */
 int _oops(char *f, int n, char *s,int err);  /* printf's and exit(1)      */
+int xfer_options(int direction, int fd);
 #define oops(s) _oops(__FILE__, __LINE__,(s),errno)
 int cmdline(int argc, char **argv);
 int giveInfo(int request, int fd);
