@@ -22,6 +22,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>        /* strerror(); ?!? memcpy() */
+#include <ctype.h>         /* isdigit */
 #include <signal.h>
 #include <stdarg.h>        /* Log uses it */
 #include <errno.h>
@@ -46,6 +47,7 @@
 
 extern int	errno;
 
+static int check_uniqueness(int retain);
 
 /*===================================================================*
  *     ==>    the logging utility
@@ -149,7 +151,7 @@ static int loadlut(char *charset)
       inwordLut[c>>5] |= 1 << (c&0x1F);
     }
   
-  if ((fd=open("/dev/console", O_WRONLY)) < 0)
+  if ((fd=open("/dev/console", O_WRONLY)) < 0) /* console is allright */
     oops("/dev/console");
   if ((fd<0) || (ioctl(fd, TIOCLINUX, &long_array)<0))
     {
@@ -165,6 +167,50 @@ static int loadlut(char *charset)
   return 0;
 }
 
+/*===================================================================*
+ * This function transfers "-a -B -p -i -l -p -r" to a running server,
+ * or gets the options from another server.
+ *-------------------------------------------------------------------*/
+int xfer_options(int direction, int fd)
+{
+Gpm_Connect fake = {0,0,0,0,getpid(),1};
+Gpm_Connect req  = {0,0,0,0, 0, GPM_REQ_CONFIG};
+struct mouse_features *which_mouse=mouse_table+1; /* local */
+char *tty;
+struct {int a,B,d,i,p,r;} opts; /* -l already processed */
+
+if (direction==0) /* out */
+  {
+  opts.a=opt_accel; sscanf(opt_sequence,"%d",&opts.B); opts.d=opt_delta;
+  opts.i=opt_time; opts.p=opt_ptrdrag; opts.r=opt_scale;
+  fd=check_uniqueness(1);
+  if (fd<0) oops(GPM_NODE_CTL);
+  tty=ttyname(0); /* stdin */
+  if (!tty) oops("ttyname()");
+  if (strncmp(tty,"/dev/tty",8) || !isdigit(tty[8]))
+    {fprintf(stderr,"%s: %s: not a virtual console\n",prgname,tty);exit(1);}
+  write(fd,&fake,sizeof(Gpm_Connect));
+  write(fd,&req,sizeof(Gpm_Connect));
+  write(fd,&opts,sizeof(opts));
+  close(fd);
+  return 0;
+  }
+if (direction==1) /* in */
+  {
+  read(fd,&opts,sizeof(opts));
+  opt_accel=opts.a;
+  sprintf(opt_sequence,"%08d",opts.B);
+  opt_delta=opts.d;
+  opt_time=opts.i; opt_ptrdrag=opts.p; opt_scale=opts.r;
+  PDEBUG((stderr,"got %i,%i,%i,%i,%i,%i\n",opt_accel,opts.B,
+	  opt_delta,opt_time,opt_ptrdrag,opt_scale));
+  kill(getpid(),SIGWINCH); /* update delta's */
+  return 0;
+  }
+return -1;
+  
+
+}
 /*===================================================================*/
 static int usage(char *whofailed)
 {
@@ -183,6 +229,8 @@ static int usage(char *whofailed)
  "    -B sequence      allows changing the buttons (default '%s')\n"
  "    -d delta         sets the delta value (default %d)\n"
  "    -D               dirty operation (debug only)\n"
+ "    -g tap-button    allows setting the button that is emulated by tapping\n"
+ "                     on a glidepoint mouse\n"
  "    -i interval      max time interval for multiple clicks (default %i)\n"
  "    -k               kill a running gpm, to start X with a busmouse\n"
  "    -l charset       loads the inword() LUT (default '%s')\n"
@@ -193,6 +241,7 @@ static int usage(char *whofailed)
  "    -o modem-lines   toggle modem lines (\"dtr\", \"rts\", \"both\")\n"
  "    -p               draw the pointer while striking a selection\n"
  "    -P               do it only with word and line drags\n"
+ "    -q               quit after changing mouse behaviour\n"
  "    -r number        sets the responsiveness (default %i)\n"
  "    -R               enter repeater mode. X should read /dev/gpmdata\n"
  "                     like it was a MouseSystem mouse device\n"
@@ -206,8 +255,11 @@ static int usage(char *whofailed)
   return 1;
 }
 
-/*===================================================================*/
-static inline void check_uniqueness(void)
+/*===================================================================*
+ * If "retain" is not 0, then a connection is open, and the fd is
+ * returned (used by "-q"). Otherwise, a check is performed
+ *-------------------------------------------------------------------*/
+static int check_uniqueness(int retain)
   {
   struct sockaddr_un ctladdr;
   int ctlfd, pid, len; FILE *f;
@@ -222,6 +274,7 @@ static inline void check_uniqueness(void)
 
   if ( connect(ctlfd,(struct sockaddr *)(&ctladdr),len)>=0 )
 	{
+	if (retain) return ctlfd;
 	/* another gpm is runnin, get its pid */
 	f=fopen(GPM_NODE_PID,"r");
 	if (f && fscanf(f,"%i",&pid)==1)
@@ -245,7 +298,8 @@ static inline void check_uniqueness(void)
 	}
 
   /* not connected, so I'm alone */
-  close(ctlfd);
+  if (retain) 
+    { close(ctlfd); return -1; }
 
   if (opt_kill)
     {
@@ -253,20 +307,20 @@ static inline void check_uniqueness(void)
     unlink(GPM_NODE_PID); /* just in case */
     exit(0);
     }
-
+return 0; /* never */
 }
 
 
 /*===================================================================*/
 int cmdline(int argc, char **argv)
 {
-char options[]="a:b:B:d:Dhi:kl:L:m:Mo:pr:Rs:t:Tv23";
+char options[]="a:b:B:d:Dg:hi:kl:L:m:Mo:pqr:Rs:t:Tv23";
 int i, opt, fd;
 FILE *f;
 static struct {char *in; char *out;} seq[] = {
   {"123","01234567"},
   {"132","02134657"},
-  {"213","01452367"},
+  {"213","01452367"}, /* warning: these must be readable as integers... */
   {"231","02461357"},
   {"312","04152637"},
   {"321","04261537"},
@@ -285,19 +339,23 @@ static struct {char *in; char *out;} seq[] = {
       case 'b': opt_baud = atoi(optarg); break;
       case 'B': opt_sequence = optarg; break;
       case 'd': opt_delta = atoi(optarg); break;
+      case 'g':
+          opt_glidepoint_tap=atoi(optarg); 
+          break;
       case 'h': exit(usage(NULL));
       case 'i': opt_time=atoi(optarg); break;
       case 'k': opt_kill++; break;
       case 'l': opt_lut=optarg; break;
-      case 'L': loadlut(optarg); exit(0);
+      case 'L': opt_lut=optarg; opt_quit++; break;
       case 'm': opt_dev = optarg; break;
       case 'M': opt_double++; which_mouse=mouse_table+2; break;
       case 'o':
           if (!strcmp(optarg,"dtr"))       opt_toggle=TIOCM_DTR;
-          else if (!strcmp(optarg,"dtr"))  opt_toggle=TIOCM_RTS;
+          else if (!strcmp(optarg,"rts"))  opt_toggle=TIOCM_RTS;
           else if (!strcmp(optarg,"both")) opt_toggle=TIOCM_DTR | TIOCM_RTS;
           else exit(usage("toggle lines"));
       case 'p': opt_ptrdrag=0; break;
+      case 'q': opt_quit++; break;
       case 'r':
 	  /* being called responsiveness, I must take the inverse */
 	  opt_scale=atoi(optarg);
@@ -316,7 +374,11 @@ static struct {char *in; char *out;} seq[] = {
       }
     }
 
-  check_uniqueness();
+  if (!opt_quit) 
+    {
+    check_uniqueness(0);
+    loadlut(opt_lut);
+    }
   if (opt_repeater)
     {
     if (mkfifo(GPM_NODE_FIFO,0666) && errno!=EEXIST)
@@ -325,7 +387,6 @@ static struct {char *in; char *out;} seq[] = {
       oops(GPM_NODE_FIFO);
     }
 
-  loadlut(opt_lut);
 
   /* duplicate initialization */
 
@@ -337,6 +398,19 @@ static struct {char *in; char *out;} seq[] = {
     if (opt_delta<2) exit(usage("delta"));
     if (strlen(opt_sequence)!=3 || atoi(opt_sequence)<100)
       exit(usage("sequence"));
+    if(opt_glidepoint_tap<1 || opt_glidepoint_tap>3)
+      exit(usage("glidepoint tap button"));
+    opt_glidepoint_tap=GPM_B_LEFT >> (opt_glidepoint_tap-1);
+
+    /* choose the sequence */
+    for (opt=0; seq[opt].in && strcmp(seq[opt].in,opt_sequence); opt++)
+      ;
+    if (!seq[opt].in) exit(usage("button sequence"));
+    opt_sequence=strdup(seq[opt].out); /* I can rewrite on it */
+
+    /* now we can put the options */
+    if (opt_quit) { exit(xfer_options(0,0)); }
+
 
     /* look for the type */
     for (m_type=mice; m_type->fun; m_type++)
@@ -379,17 +453,11 @@ static struct {char *in; char *out;} seq[] = {
       }
 
     which_mouse->fd=fd;
-
-    /* choose the sequence */
-    for (opt=0; seq[opt].in && strcmp(seq[opt].in,opt_sequence); opt++)
-      ;
-    if (!seq[opt].in) exit(usage("button sequence"));
-    opt_sequence=seq[opt].out;
     }
-  
+
 #ifndef DEBUG /* avoid forking under gdb */
 
-  /* go to background and become a session leader (Stefan Giessler) */
+   /* go to background and become a session leader (Stefan Giessler) */
   switch(fork())
     {
     case -1: oops("fork()"); /* error */
@@ -434,13 +502,3 @@ static struct {char *in; char *out;} seq[] = {
 }
   
 
-/*
- * This is because Linus uses 4-wide tabstops,
- * forcing me to use the same default to manage kernel sources
- */
-
-/* Local Variables: */
-/* tab-width:8      */
-/* End:             */
-
-
