@@ -99,7 +99,8 @@ static int magic=GPM_MAGIC;
 /*-------------------------------------------------------------------*/
 int Gpm_Open(Gpm_Connect *conn, int flag)
 {
-char *tty;
+char tty[32];
+char *term;
 int i;
 struct sockaddr_un addr;
 struct winsize win;
@@ -107,7 +108,7 @@ Gpm_Stst *new;
 
 /*....................................... First of all, check xterm */
 
-  if ((tty=(char *)getenv("TERM")) && !strncmp(tty,"xterm",5))
+  if ((term=(char *)getenv("TERM")) && !strncmp(term,"xterm",5))
     {
     if (gpm_tried) return gpm_fd; /* no stack */
     gpm_fd=-2;
@@ -118,6 +119,13 @@ Gpm_Stst *new;
 /*....................................... No xterm, go on */
 
 
+/*
+ * So I chose to use the current tty, instead of /dev/console, which
+ * has permission problems. (I am fool, and my console is
+ * readable/writeable by everybody.
+ *
+ * However, making this piece of code work has been a real hassle.
+ */
 
   if (!gpm_flag && gpm_tried) return -1;
   gpm_tried=1; /* do or die */
@@ -135,25 +143,32 @@ Gpm_Stst *new;
   else
     {
     conn->vc=0;                 /* default handler */
-    if (flag>0) conn->vc=flag;  /* forced vc number */
-    if (flag==0)  /* use your current vc */
+    if (flag>0)
+      {  /* forced vc number */
+      conn->vc=flag;
+      sprintf(tty,"/dev/tty%i",flag);
+      }
+    else if (flag==0)  /* use your current vc */
       {
-      tty=ttyname(0); /* stdin */
-      if (!tty || strncmp(tty,"/dev/tty",8) || !isdigit(tty[8]))
+      strcpy(tty,ttyname(0)); /* stdin */
+      if (strncmp(tty,"/dev/tty",8) || !isdigit(tty[8]))
 	goto err;
       conn->vc=atoi(tty+8);
       }
+    else /* a default handler -- use console */
+      sprintf(tty,"/dev/console");
+
+    if (gpm_consolefd==-1)
+      if ((gpm_consolefd=open(tty,O_WRONLY))<0)
+	{
+	PDEBUG((stderr,"%s: %s\n",tty,strerror(errno)));
+	goto err;
+	}
     }
 
   new->info=*conn;
 
 /*....................................... Get screen dimensions */
- 
-  if ((gpm_consolefd=open("/dev/console",O_WRONLY))<0)
-    {
-    PDEBUG((stderr,"/dev/console: %s\n",strerror(errno)));
-    goto err;
-    }
 
   ioctl(gpm_consolefd, TIOCGWINSZ, &win);
 
