@@ -92,6 +92,7 @@ printf("mouse: event 0x%02X, at %2i,%2i (delta %2i,%2i), "
 	 event->x, event->y,
 	 event->dx, event->dy,
 	 event->buttons, event->modifiers);
+fflush(stdout);
 
 if (opt_pointer && (event->type & GPM_DRAG))
   Gpm_DrawPointer(event->x, event->y,opt_pointer);
@@ -110,7 +111,6 @@ static char *s_button[]={"3","2","1",NULL};
 static char *s_multi[]={"double-","triple-",NULL};
 
   if (opt_fit) Gpm_FitEvent(event);
-
   buffer[0]=0;
 
   /* modifiers */
@@ -311,7 +311,7 @@ struct timeval tv;
     exit(usage());
 
   if (opt_pointer)
-    opt_pointer=open("/dev/console",O_RDWR); /* can't be 0 */
+    opt_pointer=open(ttyname(0),O_RDWR); /* can't be 0 */
 
   signal(SIGWINCH,sigHandler);
 
@@ -414,6 +414,7 @@ if (!opt_silent)
 		  case 3: printf("\tLinus Torvalds has a headache\r\n");
 		  }
 		}
+	  fflush(stdout);
 	  }
 
 
