@@ -25,6 +25,10 @@
  * issues. Each mouse type is expected to fill the `buttons', `dx' and `dy'
  * fields of the Gpm_Event structure and nothing more.
  *
+ * Absolute-pointing devices (support by Marc Meis), are expecting to
+ * fit `x' and `y' as well. Unfortunately, to do it the window size must
+ * be accessed. The global variable "win" is available for that use.
+ *
  * The `data' parameter points to a byte-array with event data, as read
  * by the mouse device. The mouse device should return a fixed number of
  * bytes to signal an event, and that exact number is read by the server
@@ -63,7 +67,7 @@ static int M_ms(Gpm_Event *state,  unsigned char *data)
    */
   static unsigned char prev=0;
 
-#if 0 /* original patch */
+#if 1 /* original patch */
 
   if (data[0] == 0x40 && !(prev|data[1]|data[2]))
     state->buttons = 2;           /* third button on MS compatible mouse */
@@ -152,9 +156,6 @@ static int M_ps2(Gpm_Event *state,  unsigned char *data)
     !!(data[0]&2) * GPM_B_RIGHT +
     !!(data[0]&4) * GPM_B_MIDDLE;
 
-/* ps2 makes me crazy.... */
-/* fprintf(stderr,"%02X %02X %02X\n",data[0],data[1],data[2]); */
-
   state->dx=   (data[0] & 0x10) ? data[1]-256 : data[1];
   state->dy= -((data[0] & 0x20) ? data[2]-256 : data[2]);
   return 0;
@@ -171,41 +172,95 @@ static int M_mman(Gpm_Event *state,  unsigned char *data)
    */
   static unsigned char prev=0;
   static Gpm_Type *mytype=mice; /* it is the first */
+  unsigned char b;
 
   if (data[1]==GPM_EXTRA_MAGIC_1 && data[2]==GPM_EXTRA_MAGIC_2)
-	{
-	/* got unexpected fourth byte */
-	LOG(("Extra byte = %02x",*data));
-	if ((*data & 0xfc) != 0x20) return -1; /* must be 0x20-0x23 */
-	state->buttons= (prev |= GPM_B_MIDDLE);
-	state->dx=state->dy=0;
-
-	mytype->packetlen=4;
-	mytype->getextra=0;
-	}
+    {
+    /* got unexpected fourth byte */
+    LOG(("Extra byte = %02x",*data));
+    if ((b=(*data>>4)) > 0x3) return -1;  /* just a sanity check */
+    state->dx=state->dy=0;
+    
+    mytype->packetlen=4;
+    mytype->getextra=0;
+    }
   else
-	{
-	/* got 3/4, as expected */
-	state->buttons= ((data[0] & 0x20) >> 3) | ((data[0] & 0x10) >> 4);
-	if (mytype->packetlen==4)
-	  {
-	  if (data[3]) state->buttons |= GPM_B_MIDDLE;
-	  else {mytype->packetlen=3; mytype->getextra=1;}
-	  }
-	}
-
-  /* motion is independent of packetlen... */
-  state->dx= (char)(((data[0] & 0x03) << 6) | (data[1] & 0x3F));
-  state->dy= (char)(((data[0] & 0x0C) << 4) | (data[2] & 0x3F));
+    {
+    /* got 3/4, as expected */
+    
+    /* motion is independent of packetlen... */
+    state->dx= (char)(((data[0] & 0x03) << 6) | (data[1] & 0x3F));
+    state->dy= (char)(((data[0] & 0x0C) << 4) | (data[2] & 0x3F));
+    
+    prev= ((data[0] & 0x20) >> 3) | ((data[0] & 0x10) >> 4);
+    if (mytype->packetlen==4) b=data[3]>>4;
+    }
+	  
+  if(mytype->packetlen==4) 
+    {
+    if(b == 0) 
+      {
+      mytype->packetlen=3;
+      mytype->getextra=1;
+      }
+    else
+      {
+      if (b & 0x2) prev |= GPM_B_MIDDLE;
+      if (b & 0x1) prev |= opt_glidepoint_tap;
+      }
+    }
+  state->buttons=prev;
 
   /* This "chord-middle" behaviour was reported by David A. van Leeuwen */
   if ( ((prev^state->buttons) & GPM_B_BOTH)==GPM_B_BOTH )
-	state->buttons = state->buttons ? GPM_B_MIDDLE : 0;
+    state->buttons = state->buttons ? GPM_B_MIDDLE : 0;
   prev=state->buttons;
   
   return 0;
 }
 
+/* ncr pen support (Marc Meis) */
+
+#define NCR_LEFT_X     40
+#define NCR_RIGHT_X    2000
+
+#define NCR_BOTTOM_Y   25
+#define NCR_TOP_Y      1490
+
+#define NCR_DELTA_X    (NCR_RIGHT_X - NCR_LEFT_X)
+#define NCR_DELTA_Y    (NCR_TOP_Y - NCR_BOTTOM_Y)
+
+static int M_ncr(Gpm_Event *state,  unsigned char *data)
+{
+  int x,y;
+
+  state->buttons= (data[0]&1)*GPM_B_LEFT +
+                !!(data[0]&2)*GPM_B_RIGHT;
+
+  state->dx = (char)data[1]; /* currently unused */
+  state->dy = (char)data[2];
+
+  x = ((int)data[3] << 8) + (int)data[4];
+  y = ((int)data[5] << 8) + (int)data[6];
+
+  /* these formulaes may look curious, but this is the way it works!!! */
+
+  state->x = x < NCR_LEFT_X
+             ? 0
+             : x > NCR_RIGHT_X
+               ? win.ws_col+1
+               : (long)(x-NCR_LEFT_X) * (long)(win.ws_col-1) / NCR_DELTA_X+2;
+
+  state->y = y < NCR_BOTTOM_Y
+             ? win.ws_row + 1
+             : y > NCR_TOP_Y
+	       ? 0
+	       : (long)(NCR_TOP_Y-y) * (long)win.ws_row / NCR_DELTA_Y + 1;
+
+  return 0;
+}
+
+
 /*========================================================================*/
 /* Then, mice should be initialized */
 
@@ -362,33 +417,36 @@ int busmouse;
 Gpm_Type mice[]={
   {"mman", "The \"MouseMan\" and similar devices (3/4 bytes per packet).",
            "Mouseman", M_mman, I_serial, CS7 | STD_FLG, /* MUST BE FIRST */
-                                {0x40, 0x40, 0x40, 0x00}, 3, 1, 1},
+                                {0x40, 0x40, 0x40, 0x00}, 3, 1, 1, 0},
   {"ms",   "The original micro$oft protocol, with a middle-button extension.",
            "", M_ms, I_serial, CS7 | STD_FLG,
-                                {0x40, 0x40, 0x40, 0x00}, 3, 1, 0},
+                                {0x40, 0x40, 0x40, 0x00}, 3, 1, 0, 0},
   {"bare", "The ms protocol, unextended. Necessary for some 2-buttons mice.",
            "Microsoft", M_bare, I_serial, CS7 | STD_FLG,
-                                {0x40, 0x40, 0x40, 0x00}, 3, 1, 0},
+                                {0x40, 0x40, 0x40, 0x00}, 3, 1, 0, 0},
   {"msc",  "Mouse-Systems-Compatible. Used in most 3-button mice (5-bytes).",
            "MouseSystems", M_msc, I_serial, CS8 | CSTOPB | STD_FLG,
-                                {0xf8, 0x80, 0x00, 0x00}, 5, 1, 0},
+                                {0xf8, 0x80, 0x00, 0x00}, 5, 1, 0, 0},
   {"sun",  "'msc' protocol, but only 3 bytes per packet.",
            "", M_sun, I_serial, CS8 | CSTOPB | STD_FLG,
-                                {0xf8, 0x80, 0x00, 0x00}, 3, 1, 0},
+                                {0xf8, 0x80, 0x00, 0x00}, 3, 1, 0, 0},
   {"mm",   "MM series. Probably an old protocol...",
            "MMSeries", M_mm, I_serial, CS8 | PARENB|PARODD | STD_FLG,
-                                {0xe0, 0x80, 0x80, 0x00}, 3, 1, 0},
+                                {0xe0, 0x80, 0x80, 0x00}, 3, 1, 0, 0},
   {"logi", "Used in some Logitech devices (only serial).",
            "Logitech", M_logi, I_logi, CS8 | CSTOPB | STD_FLG,
-                                {0xe0, 0x80, 0x80, 0x00}, 3, 3, 0},
+                                {0xe0, 0x80, 0x80, 0x00}, 3, 3, 0, 0},
   {"bm",   "Micro$oft busmice and compatible devices.",
            "BusMouse", M_bm, NULL,  STD_FLG,
-                                {0xf8, 0x80, 0x00, 0x00}, 3, 3, 0},
+                                {0xf8, 0x80, 0x00, 0x00}, 3, 3, 0, 0},
   {"ps2",  "Busmice of the ps/2 series. Most busmice, actually.",
            "PS/2", M_ps2, NULL,  STD_FLG,
-                                {0xc0, 0x00, 0x00, 0x00}, 3, 1, 0},
+                                {0xc0, 0x00, 0x00, 0x00}, 3, 1, 0, 0},
+  {"ncr",  "Ncr3125pen, found on some laptops",
+           "", M_ncr, NULL, STD_FLG,
+                                {0x08, 0x08, 0x00, 0x00}, 7, 7, 0, 1},
 
-  {"", "", "", NULL, NULL, 0,   {0x00, 0x00, 0x00, 0x00}, 0, 0, 0}
+  {"", "", "", NULL, NULL, 0,   {0x00, 0x00, 0x00, 0x00}, 0, 0, 0, 0}
 };
 
 /*------------------------------------------------------------------------*/
