@@ -48,8 +48,8 @@
 #endif
 
 #define RMEV_NAME "Mev"
-#define RMEV_RELEASE "1.00"
-#define RMEV_DATE "July 1995"
+#define RMEV_RELEASE "1.06"
+#define RMEV_DATE "November 1995"
 
 
 /*===================================================================*/
