@@ -35,9 +35,9 @@ by 't-mouse.el")
 ;;; useful for debugging
 (defun t-mouse-print (name item)
   (save-excursion
-	(set-buffer "*scratch*")
-	(end-of-buffer)
-	(insert (format "%s\t ==> %s\n" name item)))
+    (set-buffer "*scratch*")
+    (end-of-buffer)
+    (insert (format "%s\t ==> %s\n" name item)))
   (message "%s" item))
  
 
@@ -53,8 +53,10 @@ by 't-mouse.el")
       (goto-char 0)
       (or
        (re-search-forward "p \\([0-9a-f]\\)" nil t)
-       (re-search-forward "v0\\([0-9a-f]\\)"))
-      (setq tty (buffer-substring (match-beginning 1) (match-end 1))))
+       (re-search-forward "v0\\([0-9a-f]\\)" nil t))
+      (and
+       (match-beginning 1)
+       (setq tty (buffer-substring (match-beginning 1) (match-end 1)))))
     (kill-buffer buffer)
     tty
 ))
@@ -62,18 +64,21 @@ by 't-mouse.el")
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; this is the firing function
 (defun t-mouse-run ()
   (interactive)
-  (let ((process-connection-type nil))
+  (let ((process-connection-type nil) (tty (t-mouse-tty)))
     ;; remove any existing mouse process
     (and (boundp 't-mouse-process)
 	 (processp t-mouse-process)
 	 (delete-process t-mouse-process))
-    (setq t-mouse-process 
-	  ;; use only press events
-	  (start-process "t-mouse" nil "mev" "-E" "-C" (t-mouse-tty) "-M" "leftAlt"
-			 "-e" "press,drag,release,hard" "-d" "move"))
-    (set-process-filter t-mouse-process 't-mouse-process-filter)
-    (process-kill-without-query t-mouse-process)
-    t-mouse-process
+    (and
+     tty
+     (progn 
+       (setq t-mouse-process 
+	     ;; use only press events
+	     (start-process "t-mouse" nil "mev" "-E" "-C" tty "-M" "leftAlt"
+			    "-e" "press,drag,release,hard" "-d" "move" "-f"))
+       (set-process-filter t-mouse-process 't-mouse-process-filter)
+       (process-kill-without-query t-mouse-process))
+       t-mouse-process)
 ))
 
 
@@ -81,31 +86,31 @@ by 't-mouse.el")
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; this is the filter
 (defun t-mouse-process-filter (proc string)
   (let* ((event ;;(condition-case nil
-					;; protected
-					(t-mouse-make-event string)
-				  ;; protection
-				  ;;(error (progn (delete-process t-mouse-process)
-					;;	  (error "Mouse error: %s" string)))
-				  );;)
-		 (cmd (lookup-key t-mouse-keymap (car event)))
-		 (window (car (nth 1 event)))
-		 local-cmd)
-		 
-	;;(t-mouse-print "event" event)
-	;;(t-mouse-print "cmd" cmd)
-
-		 (let ((old-window (selected-window)))
-		   (or (equal (selected-window) window)
-			    (select-window window))
-		   (setq local-cmd (lookup-key t-mouse-localmap (car event)))
-		   (and local-cmd (not (numberp local-cmd)) (setq cmd local-cmd))
-			 (select-window old-window))
-
-		 (if cmd
-		     (prog2 
-			 (funcall cmd (nth 1 event))
-			 (run-hooks 'post-command-hook)))
-))
+	  ;; protected
+	  (t-mouse-make-event string)
+	  ;; protection
+	  ;;(error (progn (delete-process t-mouse-process)
+	  ;;	  (error "Mouse error: %s" string)))
+	  );;)
+	 (cmd (lookup-key t-mouse-keymap (car event)))
+	 (window (car (nth 1 event)))
+	 local-cmd)
+    
+    ;;(t-mouse-print "event" event)
+    ;;(t-mouse-print "cmd" cmd)
+    
+    (let ((old-window (selected-window)))
+      (or (equal (selected-window) window)
+	  (select-window window))
+      (setq local-cmd (lookup-key t-mouse-localmap (car event)))
+      (and local-cmd (not (numberp local-cmd)) (setq cmd local-cmd))
+      (select-window old-window))
+    
+    (if cmd
+	(prog2 
+	    (funcall cmd (nth 1 event))
+	    (run-hooks 'post-command-hook)))
+    ))
 
 
 ;;; This fun is partly Copyright (C) 1994 Per Abrahamsen <abraham@iesd.auc.dk>
@@ -118,47 +123,52 @@ by 't-mouse.el")
 	(window (window-at x y))
 	(where (coordinates-in-window-p point window))
 	(pos (if (consp where)
-			 (let ((old-window (selected-window))
-				   (dummy (select-window window))
-				   (pos (save-excursion
-						  (goto-char (window-start window))
-						  (move-to-window-line  (cdr where))
-						  (move-to-column (+ (car where) (current-column)
-											 (max 0 (1- (window-hscroll)))))
-						  (point))
-						))
-			   (select-window old-window)
-			   pos)
-		   where)))
-	;;(t-mouse-print "where" where)
+		 (let ((old-window (selected-window))
+		       (dummy (select-window window))
+		       (pos (save-excursion
+			      (goto-char (window-start window))
+			      (move-to-window-line  (cdr where))
+			      (move-to-column (+ (car where) (current-column)
+						 (max 0 (1- (window-hscroll)))))
+			      (point))
+			    ))
+		   (select-window old-window)
+		   pos)
+	       where)))
+    ;;(t-mouse-print "where" where)
     ;; manage the scroll-bar
 	
     ;; does work with horizontally-split windows
     (if t-mouse-mode
-		(prog2
-			(setq pos t-mouse-mode)
-			(if t-mouse-window (setq window t-mouse-window)))
-	  (if (and (consp where) (>= (car where) (1- (window-width window))))
-		  (setq pos 'scroll-bar)))
+	(prog2
+	    (setq pos t-mouse-mode)
+	    (if t-mouse-window (setq window t-mouse-window)))
+      (if (and (consp where) (>= (car where) (1- (window-width window))))
+	  (setq pos 'scroll-bar)))
 	
 		
     ;; make the event
     (or (integerp pos) (setq mouse (vector pos (car string))))
     (list mouse (list window pos
-					  (if (and
-						   (not (eq pos 'mode-line))
-						   (not (eq pos 'vertical-line))
-						   (consp where))
-						  where
-						point)))
+		      (if (and
+			   (not (eq pos 'mode-line))
+			   (not (eq pos 'vertical-line))
+			   (consp where))
+			  where
+			point)))
 	))
 
 
 ;; Restore normal mouse behaviour outside Emacs.
 (defun t-mouse-suspend ()
-  (process-send-string t-mouse-process "push -e 0 -d any\n"))
+  (and
+   t-mouse-process
+   (process-send-string t-mouse-process "push -e 0 -d any\n")))
+
 (defun t-mouse-resume ()
-  (process-send-string t-mouse-process "pop\n"))
+  (and
+   t-mouse-process
+   (process-send-string t-mouse-process "pop\n")))
 
 (add-hook 'suspend-hook 't-mouse-suspend)
 (add-hook 'suspend-resume-hook 't-mouse-resume)
@@ -247,13 +257,13 @@ by 't-mouse.el")
 
 (defun tm-scroll-up (event)
   ;;(message "up")
-	(select-window (car event))
+  (select-window (car event))
   (scroll-up (cdr (nth 2 event)))
 )
 
 (defun tm-scroll-down (event)
   ;;(message "down")
-	(select-window (car event))
+  (select-window (car event))
   (scroll-down (cdr (nth 2 event)))
 )
 
@@ -263,11 +273,11 @@ by 't-mouse.el")
   (let* ((lines (+ (count-lines (point-min) (point-max)) (window-height)))
 	 (target (/ (* lines (cdr (nth 2 event))) (window-height)))
 	 (step (/ lines (window-height))))
-	;;(goto-line target)
-	;;(recenter)
+    ;;(goto-line target)
+    ;;(recenter)
     (setq t-mouse-scroll (cons step  (cdr (nth 2 event))))
-		(setq t-mouse-mode 'scroll-bar)
-		(setq t-mouse-window (car event))
+    (setq t-mouse-mode 'scroll-bar)
+    (setq t-mouse-window (car event))
 ))
 
 (defun tm-scroll-drag (event)
@@ -277,8 +287,8 @@ by 't-mouse.el")
 	 (y (cdr (nth 2 event))))
     (scroll-down (* steplet (- (cdr t-mouse-scroll) y)))
     (setq t-mouse-scroll (cons step y))
-		(setq t-mouse-mode 'scroll-bar)
-		(setq t-mouse-window (car event))
+    (setq t-mouse-mode 'scroll-bar)
+    (setq t-mouse-window (car event))
 ))
 
 (defun tm-scroll-done (event)
@@ -291,12 +301,12 @@ by 't-mouse.el")
 (defun tm-win-size (event)
   (setq t-mouse-scroll (nth 2 event))
   (setq t-mouse-window
-		(let
-			((window-bottom (nth 3 (window-edges (car event))))
-			 (mini-top (nth 1 (window-edges (minibuffer-window)))))
-		  (if (eq window-bottom mini-top)
-			  (minibuffer-window)
-			(car event))))
+	(let
+	    ((window-bottom (nth 3 (window-edges (car event))))
+	     (mini-top (nth 1 (window-edges (minibuffer-window)))))
+	  (if (eq window-bottom mini-top)
+	      (minibuffer-window)
+	    (car event))))
   (setq t-mouse-mode 'mode-line))
 
 (defun tm-win-drag (event)
@@ -304,16 +314,16 @@ by 't-mouse.el")
 	(select-window (car event))
 	(let ((s-diff (- (cdr (nth 2 event)) (cdr t-mouse-scroll))))
 	  (if (equal (car event) (minibuffer-window))
-		  (shrink-window s-diff)
-		(enlarge-window s-diff)))
+	      (shrink-window s-diff)
+	    (enlarge-window s-diff)))
 	(if (and
-		 (window-live-p old-window)
-		 (window-live-p (car event)))
-		(progn
-		  (setq t-mouse-scroll (nth 2 event))
-		  (setq t-mouse-window (car event))
-		  (setq t-mouse-mode 'mode-line)
-		  (select-window old-window))
+	     (window-live-p old-window)
+	     (window-live-p (car event)))
+	    (progn
+	      (setq t-mouse-scroll (nth 2 event))
+	      (setq t-mouse-window (car event))
+	      (setq t-mouse-mode 'mode-line)
+	      (select-window old-window))
 	  (tm-scroll-done event))))
 
 
@@ -325,18 +335,18 @@ by 't-mouse.el")
 
 (defun tm-win-drag-horizontally (event)
 	(let ((old-window (selected-window)))
-		(select-window (car event))
-		(let ((s-diff (- (car (nth 2 event)) (car t-mouse-scroll))))
-		  (enlarge-window-horizontally s-diff))
-		(if (and
-		     (window-live-p old-window)
-		     (window-live-p (car event)))
-		    (progn
-		      (setq t-mouse-scroll (nth 2 event))
-		      (setq t-mouse-window (car event))
-		      (setq t-mouse-mode 'vertical-line)
-		      (select-window old-window))
-		  (tm-scroll-done event))))
+	  (select-window (car event))
+	  (let ((s-diff (- (car (nth 2 event)) (car t-mouse-scroll))))
+	    (enlarge-window-horizontally s-diff))
+	  (if (and
+	       (window-live-p old-window)
+	       (window-live-p (car event)))
+	      (progn
+		(setq t-mouse-scroll (nth 2 event))
+		(setq t-mouse-window (car event))
+		(setq t-mouse-mode 'vertical-line)
+		(select-window old-window))
+	    (tm-scroll-done event))))
 
 ;;; and then conventional events
 
@@ -379,8 +389,8 @@ by 't-mouse.el")
   
 (defun tm-line (event)
   (let* ((current (nth 1 event))
-	(previous (mark))
-	(backward (> previous current)))
+	 (previous (mark))
+	 (backward (> previous current)))
     (tm-drag event)
     (pop-mark)
     ;; update previous and set mark
@@ -403,37 +413,37 @@ by 't-mouse.el")
     (yank)))
 
 (defun tm-copy (event)
-	(select-window (car event))
+  (select-window (car event))
   (let ((current (nth 1 event)))
     (kill-ring-save (mark) current)
     (goto-char current)))
 
 (defun tm-kill (event)
-	(select-window (car event))
+  (select-window (car event))
   (let ((current (nth 1 event)))
     (kill-region (mark) current)))
 
 (defun tm-win-single (event)
-	(select-window (car event))
+  (select-window (car event))
   (delete-other-windows))
 
 (defun tm-win-delete (event)
-	(select-window (car event))
+  (select-window (car event))
   (if (one-window-p 'no-mini)
       (kill-buffer (window-buffer (car event)))
     (delete-window)))
 
 (defun tm-win-split (event)
-	(let ((old-window (selected-window)))
-		(select-window (car event))
-		(split-window)
-		(select-window old-window)))
+  (let ((old-window (selected-window)))
+    (select-window (car event))
+    (split-window)
+    (select-window old-window)))
 
 (defun tm-win-split-horizontally (event)
-	(let ((old-window (selected-window)))
-		(select-window (car event))
-		(split-window-horizontally)
-		(select-window old-window)))
+  (let ((old-window (selected-window)))
+    (select-window (car event))
+    (split-window-horizontally)
+    (select-window old-window)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; This is mode-specific code
 
@@ -488,24 +498,24 @@ by 't-mouse.el")
 	  (tm-goto event)
 	  (setq cwindow (selected-window))
 	  (let (beg end)
-		(skip-chars-forward "^ \t\n")
-		(while (looking-at " [^ \n\t]")
-		  (forward-char 1)
-		  (skip-chars-forward "^ \t\n"))
-		(setq end (point))
-		(skip-chars-backward "^ \t\n")
-		(while (and (= (preceding-char) ?\ )
-					(not (and (> (point) (1+ (point-min)))
-							  (= (char-after (- (point) 2)) ?\ ))))
-		  (backward-char 1)
-		  (skip-chars-backward "^ \t\n"))
-		(setq beg (point))
-		(setq choice (buffer-substring beg end))))
+	    (skip-chars-forward "^ \t\n")
+	    (while (looking-at " [^ \n\t]")
+	      (forward-char 1)
+	      (skip-chars-forward "^ \t\n"))
+	    (setq end (point))
+	    (skip-chars-backward "^ \t\n")
+	    (while (and (= (preceding-char) ?\ )
+			(not (and (> (point) (1+ (point-min)))
+				  (= (char-after (- (point) 2)) ?\ ))))
+	      (backward-char 1)
+	      (skip-chars-backward "^ \t\n"))
+	    (setq beg (point))
+	    (setq choice (buffer-substring beg end))))
 	;;(t-mouse-print "choice" choice)
 	(select-window cwindow)
 	(bury-buffer)
 	(select-window owindow)
-    (choose-completion-string choice buffer)))
+	(choose-completion-string choice buffer)))
 
 
 ;;;    These files use "mouse" with "keymap"
